/*****************************************************************************/
/*  MATRIXDEMO.CS   MS-WINDOWS Win32 (95/98/ME/NT/2K/XP)                     */
/*                                                                           */
/*  (C) TDi GmbH                                                             */
/*                                                                           */
/*  Example for C#                                                           */
/*****************************************************************************/

using System;
using System.Windows.Forms; // MessageBox
using System.Runtime.InteropServices; // Marshalling
using System.Text;
using MXAPI;

namespace matrix
{
	/// <summary>
	/// Summary description for MatrixDemo.
	/// </summary>
	public class MatrixDemo
	{
		private short RetCode;
		private int[] DataIn;
		private int[] DataOut;
		private int[] KeyData;
		private int[] DataBlock;
		private short DNG_Count;
		private short DNG_Nummer;
		private short DNG_Mem;
		private short DNG_MaxVar;
		private int DNG_Version;
		private int DNG_SerNr;
		private short VerMajor;
		private short VerMinor;
		private short AppSlot;
	
		// ***********************************************************
		// * In this example we use the demo UserCode 1234.          *
		// ***********************************************************
		const int UserCode = 1234;
		short DNG_Port;

		public MatrixDemo()
		{
			DataIn = new int[256];
			DataOut = new int[256];
			KeyData = new int[4];
            DataBlock = new int[2];
        
            DNG_Nummer = 1;
        }


		public void SetActivePort(string Port)
		{
			DNG_Port = 0;
			switch(Port)
			{
				case "USB":
					DNG_Port = 85;
					break;
			}    
		}

        public void Number_of_Dongles()
		{
			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Search for number of Dongles at DNG_Port                *
			// ***********************************************************
			DNG_Count = Matrix.Dongle_Count(DNG_Port);
		
			if(DNG_Count == 0)
				MessageBox.Show("hO܂");
			else if(DNG_Count < 0)
                MessageBox.Show("G[:" + DNG_Port.ToString() );
            else
				MessageBox.Show("hO " + DNG_Count.ToString() + " ܂");
		
			RetCode = Matrix.Release_MatrixAPI();
		}


		public void Memory_of_Dongle()
		{
			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Read memory size of last Dongle at DNG_Port and         *
			// * calculate the maximum number of data fields             *
			// ***********************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
			DNG_Mem = Matrix.Dongle_MemSize(DNG_Nummer, DNG_Port);
		
			if(DNG_Mem <= 0)
			{
				MessageBox.Show("MemSizeG[:" + DNG_Mem.ToString());
				return;
			}
		
			DNG_MaxVar = (short)(DNG_Mem / 4);
		
			MessageBox.Show("f[^tB[h: " + DNG_MaxVar.ToString());	
			
			RetCode = Matrix.Release_MatrixAPI();
		}
		
		public void Version_of_Dongle()
		{
			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Read Dongle version from last Dongle at DNG_Port        *
			// ***********************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
			DNG_Version = Matrix.Dongle_Version(DNG_Nummer, DNG_Port);
		
			if(DNG_Version <= 0)
			{
				MessageBox.Show("VersionG[:" + DNG_Version.ToString());
			}
			else
			{
				VerMinor = (short)(DNG_Version & 0xffff);
				VerMajor = (short)(DNG_Version >> 16);
				MessageBox.Show("o[W:" + VerMajor.ToString() + "." + VerMinor.ToString());
			}

			RetCode = Matrix.Release_MatrixAPI();		
		}
	
		public void Read_Dongle_SerNr()
		{
			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Read the SerialNo. of last Dongle at DNG_Port via       *
			// * UserCode and display.                                   *
			// ***********************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
			DNG_SerNr = Matrix.Dongle_ReadSerNr(UserCode, DNG_Nummer, DNG_Port);
		
			if(DNG_SerNr < 0)
			{
				MessageBox.Show("G[:" + DNG_SerNr.ToString() );
				RetCode = Matrix.Release_MatrixAPI();
				return;
			}
			MessageBox.Show("VAԍ: (10i) " + DNG_SerNr.ToString() + "    (16i) " + DNG_SerNr.ToString("X"));
		
			RetCode = Matrix.Release_MatrixAPI();
		
		}
	
        
        // Allocate unmanaged memory to hold the integer array with the given length
        private IntPtr getIntPtr( int[] Data, int length)
        {
            int size = Marshal.SizeOf(Data[0]) * length;
            return  Marshal.AllocHGlobal(size);
        }

		public void Read_Dongle_Data()
		{
			string MessageString;
			short  i;
            short copy_count = 3;

			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Read 3 data fields from last Dongle at DNG_Port via     *
			// * UserCode and display.                                   *
			// ***********************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
            IntPtr pnt = getIntPtr(DataIn, 3 );

            try
            {
                RetCode = Matrix.Dongle_ReadData(UserCode, pnt, copy_count, DNG_Nummer, DNG_Port);

                // Copy unmanged integer arry values to the managed integer array
                if( RetCode > 0)
                    Marshal.Copy(pnt, DataIn, 0, 3);

            }
            finally
            {
                Marshal.FreeHGlobal(pnt);
            }


            if (RetCode < 0)
            {
                MessageBox.Show("G[:" + RetCode.ToString());
                RetCode = Matrix.Release_MatrixAPI();
                return;
            }

			MessageString = "ǂݍݒl: \n\n";
			for(i = 0; i <= 2; i++)
			{
				MessageString = MessageString + "tB[h:" + (i + 1).ToString("D4") + ":  " + DataIn[i].ToString() + "\n";
			}
			MessageBox.Show(MessageString);
		
			RetCode = Matrix.Release_MatrixAPI();
		
		}
	
		public void Write_Dongle_Data()
		{
			short  i;
            short copy_count = 3;

			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Save 3 data fields in last Dongle at DNG_Port via       *
			// * UserCode. The values 101,102,103 are saved as example.  *
			// ***********************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);

            for (i = 0; i < copy_count; i++)
			{
				DataOut[i] = 101 + i;
			}

            // Initialize unmanaged memory to hold the array.
            IntPtr pnt = getIntPtr(DataOut, 3);

            try
            {
                // Copy the array to unmanaged memory.
                Marshal.Copy(DataOut, 0, pnt, copy_count);

                RetCode = Matrix.Dongle_WriteData(UserCode, pnt, copy_count, DNG_Nummer, DNG_Port);

                if( RetCode > 0)
                    Marshal.Copy(pnt, DataOut, 0, copy_count);

            }
            finally
            {
                Marshal.FreeHGlobal(pnt);
            }


            if (RetCode < 0)
            {

                MessageBox.Show("G[:" + RetCode.ToString());
                RetCode = Matrix.Release_MatrixAPI();
                return;
            }
            
            MessageBox.Show("ݐ");
	
			RetCode = Matrix.Release_MatrixAPI();		
		}	

		public void Dongle_Encrypt_Decrypt()
		{
			string MessageString;
		
			RetCode = Matrix.Init_MatrixAPI();
		
			DataBlock[0] = 1234567890; // Clear Data
			DataBlock[1] = 1234567890; // Clear Data
		
			MessageString = ":   \t" + "(10i)  " + DataBlock[0].ToString() + "\t   " + DataBlock[1].ToString() + "\n\t\t(16i)  " + DataBlock[0].ToString("X") + "\t   " + DataBlock[1].ToString("X") + "\n\n";
		
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
			// ***********************************************************
			// * Encrypt DataBlock over the Dongle                       *
			// ***********************************************************

            IntPtr pnt = getIntPtr( DataBlock, 2 );

            try
            {
                // Copy the array to unmanaged memory.
                Marshal.Copy(DataBlock, 0, pnt, 2);

                RetCode = Matrix.Dongle_EncryptData(UserCode, pnt, DNG_Nummer, DNG_Port);

                if (RetCode > 0)
                    Marshal.Copy(pnt, DataBlock, 0, 2);

            }
            finally
            {
                Marshal.FreeHGlobal(pnt);
            }


		
			if(RetCode < 0)
			{
				MessageBox.Show("G[:" + RetCode.ToString());
				RetCode = Matrix.Release_MatrixAPI();
				return;
			}
		
			MessageString = MessageString + "Íf[^: \t(10i)  " + DataBlock[0].ToString() + "\t   " + DataBlock[1].ToString() + "\n\t\t(16i)  " + DataBlock[0].ToString("X") + "\t   " + DataBlock[1].ToString("X") + "\n\n";
		
			// ***********************************************************
			// * Decrypt DataBlock over the Dongle                       *
			// ***********************************************************
            pnt = getIntPtr( DataBlock, 2);

            try
            {
                // Copy the array to unmanaged memory.
                Marshal.Copy(DataBlock, 0, pnt, 2);

                RetCode = Matrix.Dongle_DecryptData(UserCode, pnt, DNG_Nummer, DNG_Port);

                if (RetCode > 0)
                    Marshal.Copy(pnt, DataBlock, 0, 2);

            }
            finally
            {
                Marshal.FreeHGlobal(pnt);
            }
            
		
			if(RetCode < 0)
			{
				MessageBox.Show("G[F" + RetCode.ToString());
				RetCode = Matrix.Release_MatrixAPI();
			}
		
			MessageBox.Show(MessageString + "f[^: \t(10i)  " + DataBlock[0].ToString() + "\t   " + DataBlock[1].ToString() + "\n\t\t(16i)  " + DataBlock[0].ToString("X") + "\t   " + DataBlock[1].ToString("X") + "\n\n");
	
			RetCode = Matrix.Release_MatrixAPI();
		}
	
		public void Version_of_API()
		{
			int RetVer;
		
			RetCode = Matrix.Init_MatrixAPI();
		
			// ***********************************************************
			// * Read the version of the Matrix-API                      *
			// ***********************************************************
			RetVer = Matrix.GetVersionAPI();
		
			if(RetVer == 0)
				MessageBox.Show("APIo[WG[:" + RetVer.ToString());
			else
			{
				VerMinor = (short)(RetVer & 0xffff);
				VerMajor = (short)(RetVer >> 16);
				MessageBox.Show("Matrix-APIo[W: " + VerMajor.ToString() + "." + VerMinor.ToString());
			}		
		
			RetCode = Matrix.Release_MatrixAPI();
		
		}
	
		public void Dongle_Network()
		{

			// *************************************************************
			// * Network                                                   *
			// * Example: the dongle is prepared with the value 7 in       *
			// *          Var0002, meaning AppSlot=2 and max. Users = 7    *
			// *          (AppSlot2 = max. 7 Users)                        *
			// *************************************************************
		
			RetCode = Matrix.Init_MatrixAPI();
		
			// *************************************************************
			// * Max. Users stored in AppSlot 2 (Var0002 of the dongle)    *
			// *************************************************************
			AppSlot = 2;
		
			// *************************************************************
			// * Activate Network access into the Matrix-API               *
			// *************************************************************
			string MxNetFile = "c:\\matrix\\test.svr";

			RetCode = Matrix.SetConfig_MatrixNet(1, MxNetFile);
		
			// *************************************************************
			// * Read the Dongle count from MxNetFile                      *
			// * !! DNG_Port will be ignored in the Network mode !!        *
			// *************************************************************
			DNG_Nummer = Matrix.Dongle_Count(DNG_Port);
		
			// *************************************************************
			// * LogIn will lock one User-Slot and the returned value is   *
			// * the remaining free User-Slots (for our Example RetCode=6) *
			// *************************************************************
			RetCode = Matrix.LogIn_MatrixNet(UserCode, AppSlot, DNG_Nummer);
			if(RetCode < 0)
			{
				if(RetCode == -31)
					MessageBox.Show("CZXcȂ");
				else
					MessageBox.Show("LogInG[:" + RetCode.ToString());
			}	
			else
				MessageBox.Show("LogIn !  CZXc: " + RetCode.ToString() );
		
			// *************************************************************
			// * LogOut will free the User-Slot and the returned value is  *
			// * the remaining free User-Slots (for our Example RetCode=7) *
			// *************************************************************
			RetCode = Matrix.LogOut_MatrixNet(UserCode, AppSlot, DNG_Nummer);
			if(RetCode < 0)
				MessageBox.Show("LogOut G[:" + RetCode.ToString());
			else
				MessageBox.Show("LogOut! CZXc: " + RetCode.ToString());
		
			// *************************************************************
			// * Deactivate Network access into the Matrix-API             *
			// *************************************************************
			RetCode = Matrix.SetConfig_MatrixNet(0, null);
		
			RetCode = Matrix.Release_MatrixAPI();
		
		}
	
		public void App_Encrypt()
		{
			string MessageString;
			uint[] Key = new uint[4];
			uint[] Data = new uint[2];
		
			Data[0] = 0x499602D2;   // dec: 1234567890
			Data[1] = 0x499602D2;   // dec: 1234567890
		
			Key[0] = 0x423A612E;    // dec: 1111122222
			Key[1] = 0x423A8C95;    // dec: 1111133333
			Key[2] = 0x8474C25C;    // dec: 2222244444
			Key[3] = 0x8474EDC3;    // dec: 2222255555
		
			MessageString = ":   \t(10i)  " + Data[0].ToString() + "\t      " + Data[1].ToString() + 
			                          "\n\t\t(16i)  " + Data[0].ToString("X") + "\t      " + Data[1].ToString("X") + 
			                          "\n\n" + 
			                "Í:  \t(10i)  " + Key[0].ToString() + "\t      " + Key[1].ToString() + "\t  " + Key[2].ToString() + "\t" + Key[3].ToString() + 
			                          "\n\t\t(16i)  " + Key[0].ToString("X") + "\t      " + Key[1].ToString() + "\t  " + Key[2].ToString() + "\t" + Key[3].ToString("X") + "\n\n";

			MATRIX_ENCRYPT.MxCrypt.MxApp_Encrypt(Data, Key);
		
			// *** Display Data[0]/Data[1] as long ***
			MessageBox.Show(MessageString + "Íf[^: \t(10i)  " + Data[0].ToString() + "\t      " + Data[1].ToString() + 
			                                            "\n\t\t(16i)  " + Data[0].ToString("X") + "\t      " + Data[1].ToString("X") + "\n\n");
		}

	} // end class
} // end namespace

